#!/usr/bin/perl -w
use strict;

my $truthFN = shift or die "usage: evaluate.pl [truth-file] [system-file]\n";
my $systemFN = shift or die "usage: evaluate.pl [truth-file] [system-file]\n";

my $count = 0;
my $errors = 0;
my %examples = ();

my %tags = ();
my %conf = ();

my $line = 0;
open T, $truthFN or die "cannot open $truthFN\n";
open S, $systemFN or die "cannot open $systemFN\n";
while (<T>) {
    $line++;
    chomp;
    my @t = split;
    $_ = <S>;
    if (not defined $_) { die "$systemFN is not as long as $truthFN\n"; }
    chomp;
    my @s = split;
    
    my $err = 0;
    if (scalar @s != scalar @t) { die "[line $line] length mismatch\n"; }
    for (my $i=0; $i<@s; $i++) {
        my $s = $s[$i];
        if ($s =~ /^.+_([^_]+)$/) { $s = $1; }
        my $t = $t[$i];
        if ($t =~ /^.+_([^_]+)$/) { $t = $1; }
        if ($s ne $t) { $err++; }
        $conf{$t}{$s}++;
        $tags{$t} = 1;
        $tags{$s} = 1;
    }
    $count += scalar @s;
    $errors += $err;
    $examples{$err/scalar @s}{join "\t", @t} = join "\t", @s;
}
close T or die;
while (<S>) { die "$truthFN not as long as $systemFN\n"; }
close S or die;

print "Accuracy: " . (int(10000 - 10000 * $errors / $count)/100) . "%\n";
print "\n";
print "Confusion Matrix:\n";
print "\t\t\t\t\tTRUTH\n";
foreach my $t (sort keys %tags) {
    print "\t$t";
}
print "\n";
foreach my $s (sort keys %tags) {
    print $s;
    foreach my $t (sort keys %tags) {
        print "\t";
        if ((exists $conf{$t}) && (exists $conf{$t}{$s})) {
            print $conf{$t}{$s};
        } else {
            print 0;
        }
    }
    print "\n";
}
print "\n";
print "Sentences with many errors:\n";
$count = 10;
foreach my $err (sort { $b <=> $a } keys %examples) {
    if ($err <= 0) { last; }
    foreach my $s (keys %{$examples{$err}}) {
        if ($count <= 0) { last; }
        $count--;
        print '' . (int($err*1000)/100) . "\t" . $s . "\n";
        print "\t" . $examples{$err}{$s} . "\n";
    }
    if ($count <= 0) { last; }
}

if ($count == 10) { print "None!  Great job!\n"; }
